export TAPPER_REPORT_SERVER=localhost    # default in tools: 'tapper'
export TAPPER_WEB_PORT=3000              # default in tools: '80'
#export TAPPER_REPORT_PORT=7357          # default in tools: '7357'
#export TAPPER_API_PORT=7358             # default in tools: '7358'
export BENCHMARKANYTHING_CONFIGFILE=__HOME__/.tapper/tapper.cfg
